/*
 * ATTENTION ! 
 * Cette classe implémente le moteur de jeu
 * Il n'est pas nécessaire de modifier cette classe pour le TP !
 */
package spaceinvaders;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.Timer;

/**
 * Moteur de jeux pour Space invaders.
 * 
 * @author guillaume
 */
@SuppressWarnings("serial")
public abstract class GameEngine extends Frame {

	/** Liste des objets du jeu */
	final ArrayList<GameObject> _game_objects = new ArrayList<GameObject>();

	/** Permet d'effecteur le rendu avant de l'afficher : scintille pas */
	private Graphics _backBuffer = null;
	private BufferedImage _backBufferImage = null;

	/** Ecoute les evenements clavier */
	private GameKeyAdapter _gameKeyAdapter = null;

	/** Permet de demander l'update graphique et logique à intervalles donnés */
	private Timer _timerUpdate = null;

	/** Permet de savoir si l'initialisation du moteur est terminée */
	private boolean _init_done = false;

	/** Constantes taille d'écran */
	public final static int _HEIGHT_SCREEN = 600;
	public final static int _WIDTH_SCREEN = 800;

	/** Constante refraichissement, en images par seconde */
	public final static int _FPS = 60;

	/** Si le jeu est perdu */
	private boolean _gameOver = false;

	/**
	 * Constructeur du moteur de jeu
	 */
	public GameEngine() {
	}

	/**
	 * A redefinir pour chaque jeu : appelé à chaque fois qu'une touche est
	 * pressée
	 * 
	 * @param code
	 *            le code de la touche (voir KeyEvent.VK_*)
	 */
	public abstract void traiteTouche(int code);

	/**
	 * A redefinir, permet de dire qu'un certain temps s'est écoulé et qu'il
	 * faut dont effectuer certaines mises à jour
	 * 
	 * @param time
	 *            temps écoulé
	 */
	public abstract void updateGame(int time);

	/**
	 * Permet d'ajouter un objet au moteur, pour qu'il soit dessiné puis updaté
	 * 
	 * @param object
	 *            l'objet a ajouter
	 */
	public final void addGameObject(GameObject object) {
		_game_objects.add(object);
	}

	/**
	 * Permet de retirer un objet au moteur, pour qu'il ne soit plus dessiné ou
	 * updaté
	 * 
	 * @param object
	 *            l'objet a enlever
	 */
	public final void removeGameObject(GameObject object) {
		_game_objects.remove(object);
	}

	/**
	 * Définit si le jeu est terminé ou pas
	 * 
	 * @param gameover
	 *            true si jeu terminé
	 */
	public final void setGameOver(boolean gameover) {
		_gameOver = gameover;
	};

	/**
	 * Permet de savoir si le jeu est terminé ou pas
	 * 
	 * @return true si jeu terminé
	 */
	public final boolean getGameOver() {
		return _gameOver;
	};

	/**
	 * Demarre le moteur de jeu
	 */
	public final void startEngine() {
		// Taille de la fenetre
		this.setSize(_WIDTH_SCREEN, _HEIGHT_SCREEN);

		// On écoute l'evenement de fermeture de fenetre, pour effectivement
		// fermer la fenetre
		this.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent we) {
				System.exit(0);
			}
		});

		// On crée le buffer de rendu
		_backBufferImage = new BufferedImage(_WIDTH_SCREEN, _HEIGHT_SCREEN,
				BufferedImage.TYPE_INT_RGB);
		_backBuffer = _backBufferImage.getGraphics();

		// Ecouteur de touches
		_gameKeyAdapter = new GameKeyAdapter();
		addKeyListener(_gameKeyAdapter);

		// On affiche la fenetre
		this.setVisible(true);

		// Lance l'update
		_timerUpdate = new Timer(1000 / _FPS, new UpdateTimerTask());
		_timerUpdate.start();

		// Init terminée
		_init_done = true;
	}

	/**
	 * Fonction de rendu
	 */
	private final void render() {
		repaint();
	}

	/**
	 * Methode de base de la frame pour le rendu
	 * 
	 * @param g
	 *            Le fond graphique ou l'on peut dessiner
	 */
	@Override
	public final void paint(Graphics g) {
		// On ne rend que si l'init est terminée
		// if(!_init_done)
		// return; (Inutile, on utilise les timers swing ici.)

		// On efface le backbuffer
		_backBuffer.clearRect(0, 0, _WIDTH_SCREEN, _HEIGHT_SCREEN);

		// On peint les objets du jeu

		Iterator<GameObject> it = _game_objects.iterator();
		while (it.hasNext()) {
			GameObject go = it.next();
			go.draw(_backBuffer);
		}

		// On passe le backbuffer à l'écran
		g.drawImage(_backBufferImage, 0, 0, null);
	}

	/**
	 * Permet d'éviter l'effacage de l'écran avant le rendu (sinon scintille)
	 * 
	 * @param g
	 *            la ou il faut dessiner pour modifier la fenetre
	 */
	@Override
	public final void update(Graphics g) {
		paint(g);
	}

	/**
	 * Tache timer pour avoir un update régulier des objets et rafraichir
	 * l'écran
	 */
	private class UpdateTimerTask implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent ae) {
			if (!_init_done)
				return;

			if (getGameOver())
				return;

			updateObjects(1000 / _FPS);
			updateGame(1000 / _FPS);
			render();
		}
	}

	/**
	 * Permet de dire à tous les objets du jeu qu'un certain temps est passé et
	 * qu'il faut se mettre à jour.
	 * 
	 * @param elapsed
	 *            le temps écoulé depuis la dernière mise à jour
	 */
	private void updateObjects(int time) {
		// On update les objets du jeu.
		// On utilise une boucle classique et non un itérateur pour
		// éviter la création d'objet.
		for (int i = 0; i < _game_objects.size(); i++) {
			GameObject go = _game_objects.get(i);
			go.update(time);
		}
	};

	/**
	 * Permet d'écouter les evenements touches
	 */
	private class GameKeyAdapter extends KeyAdapter {
		@Override
		public void keyPressed(KeyEvent e) {
			// On ne traite que si init terminé
			if (!_init_done)
				return;
			int code = e.getKeyCode();
			traiteTouche(code);
		}
	}

	/**
	 * Permet de simplifier le chargement d'une image. Par contre, demande un
	 * dossier images à coté du .jar ou dans le dossier du projet netbeans
	 * 
	 * @param name
	 *            juste le nom du fichier a charger (dans le dossier images)
	 * @return une image, en cours de chargement.
	 */
	public static Image loadImage(String name) {
		Image image = null;
		URL url_image = GameEngine.class.getResource(name);

		try {
			image = Toolkit.getDefaultToolkit().getImage(url_image);
		} catch (Exception e) {
			System.out.println("Erreur image invader : " + e.getMessage());
		}
		return image;
	}
}
