/*
 * 
 */
package spaceinvaders;

import java.awt.*;

/**
 * Un objet du jeu
 * @author guillaume
 */
public class GameObject{
    
    /**Pour afficher l'objet*/
    private Image _image = null;
    
    /**Coordonnées de l'objet*/
    private int _x = 0;
    private int _y = 0;
    
    /**Permet de poser des contraintes maximales et minimales à l'objet*/
    protected int _min_x = 0;
    protected int _min_y = 0;
    protected int _max_x = 0;
    protected int _max_y = 0;
    protected boolean _secure_x = false;
    protected boolean _secure_y = false;
    
    /**Si l'objet et toujours en vie (sinon on ne l'affiche pas)*/    
    protected boolean _alive = true;
    
    /**Boite de collision*/
    private Rectangle _bbox;
    private int _offsetBbox_x = 0;
    private int _offsetBbox_y = 0;
 
    /**
     * Contructeur
     */
    public GameObject(){
        _bbox = new Rectangle(_x, _y, 1, 1);
    }
    
    /**Accesseurs*/
    public int getY(){return _y;};
    public int getX(){return _x;};
    
    /**
     * L'objet est-il vivant.
     * @return vrai si l'objet est vivant.
     */
    public boolean isAlive(){return _alive;};
    
    /**
     * Change le statut de l'objet vivant ou non.
     * Seuls les objets vivants sont dessinés et pris en compte pour les collisions.
     * En revanche, la méthode "update" est appelée sur <em>tous</em> les objets, vivants ou non.
     * @param alive
     */
    public void setAlive(boolean alive){_alive = alive;};
    
    public Rectangle getBox(){return _bbox;};
    
 
    /**
     * Permet de choisir l'image a afficher
     * @param filename juste le nom de l'image
     */
    public void setImage(String filename){
        _image = GameEngine.loadImage(filename);
    }
    
    /**
     * Appelé par le moteur de jeu pour demander à l'objet de se dessiner
     * @param graphic le contexte graphique sur leque l'objet doit se dessiner
     */
    public void draw(Graphics graphic){
        if(_image != null  && _alive)
            graphic.drawImage(_image, _x, _y, null);
    }     
        
    /**
     * Test de collision avec une boite de collision rectangle donnée
     * @param other une boite de collision sous forme de rectangle
     * @return true si collision
     */
    public boolean collideWith(Rectangle other){
        return other.intersects(_bbox) && _alive;
    }
    
    
    /**
     * A redéfinir : appelée par le moteur de jeu pour mettre à jour l'objet
     * @param time le temps écoulé depuis le dernier appel.
     */
    public void update(int time){
        
    }
    
    /**
     * Deplace l'objet de n pixels, relativement à sa position actuelle
     * Utiliser ces fonctions permet de maintenir la cohérence avec la bbox
     * @param x déplacement horizontal
     * @param y déplacement vertical
     */
    public void moveRelative(int x, int y){
        _x += x;
        _y += y;
        securePosition();
        updateBox();
    }
    
    /**
     * Permet un déplacement absolu de l'objet
     * Utiliser ces fonctions permet de maintenir la cohérence avec la bbox
     * @param x nouvelle position
     * @param y nouvelle position
     */
    public void moveTo(int x, int y){
        _x = x;
        _y = y;
        securePosition();
        updateBox();
    }
    
    /**
     * Permet de définir la bbox
     * @param offsetX
     * @param offsetY 
     */
    protected void setBbox(int offsetX, int offsetY, int width, int height){
        _bbox.x = _x + _offsetBbox_x;
        _bbox.y = _y + _offsetBbox_y;
        _bbox.width = width;
        _bbox.height = height;
    }
            
    
    /**
     * Modifie la bbox à chaque déplacement de l'objet
     * A redefinir car chaque objet a sa propre box
     */
    private void updateBox(){
        _bbox.x= _x + _offsetBbox_x;
        _bbox.y= _y + _offsetBbox_y;
    }   
    
    /**
     * Permet de poser des limites aux coordonnées
     * @param securex si l'objet doit s'imposer des limites horizontales
     * @param minx limite min x
     * @param maxx limite max x
     * @param securey si l'objet doit s'imposer des limites verticales
     * @param miny limite min y
     * @param maxy limite max y
     */
    protected void setCoordsLimits(boolean securex,int minx, int maxx, boolean securey, int miny, int maxy){
        _secure_x = securex;
        _secure_y = securey;
        _min_x = minx;
        _max_x = maxx;
        _min_y = miny;
        _max_y = maxy;
    }
    
    /**
     * Contraint l'objet à respecter les bornes horizontales et verticales
     */
    private void securePosition(){
        if(_x < _min_x && _secure_x)
            _x = _min_x;
        if(_x > _max_x && _secure_x)
            _x = _max_x;
        if(_y < _min_y && _secure_y)
            _y = _min_y;
        if(_y > _max_y && _secure_y)
            _y = _max_y;
    }              
}
